#ifndef PROFILE_H
#define PROFILE_H

#include <QObject>
#include <QTimer>

#include "model.h"
#include "connection.h"
#include "proxymodel.h"
#include "cache.h"

#define DEFAULT_GET_MAIL_INTERVAL 5 * 60 * 1000

namespace FlyMail {

class FLYMAILSHARED_EXPORT Profile : public QObject
{
    Q_OBJECT
    Q_PROPERTY(int          unseenLetters   MEMBER m_unseenLetters      NOTIFY unseenLettersChanged)
    Q_PROPERTY(QString      text            MEMBER m_email              NOTIFY emailChanged) // for convenience
    Q_PROPERTY(QString      name            MEMBER m_name               NOTIFY nameChanged)
    Q_PROPERTY(QString      email           MEMBER m_email              NOTIFY emailChanged)
    Q_PROPERTY(QString    incomingServerStr MEMBER m_incomingServerStr)
    Q_PROPERTY(QString    outgoingServerStr MEMBER m_outgoingServerStr)
    Q_PROPERTY(ProxyModel * incomingModel   MEMBER m_incomingProxyModel NOTIFY incomingProxyModelChanged)
    Q_PROPERTY(ProxyModel * draftsModel     MEMBER m_draftsProxyModel   NOTIFY draftsProxyModelChanged)
    Q_PROPERTY(ProxyModel * sentModel       MEMBER m_sentProxyModel     NOTIFY sentProxyModelChanged)
    Q_PROPERTY(ProxyModel * archiveModel    MEMBER m_archiveProxyModel  NOTIFY archiveProxyModelChanged)
    Q_PROPERTY(ProxyModel * trashModel      MEMBER m_trashProxyModel    NOTIFY trashProxyModelChanged)
    Q_PROPERTY(Cache      * cache           MEMBER m_cache)
    Q_PROPERTY(Letter     * newLetter       MEMBER m_newLetter)
    Q_PROPERTY(Status       status          MEMBER m_status             NOTIFY statusChanged)

public:
    enum Status { READY_STATUS, RECIEVING_STATUS, SENDING_STATUS };
    Q_ENUM(Status)

    explicit Profile(QString email = "", QObject *parent = nullptr);
    ~Profile();
    void initialize();
    void saveToSettings();

    bool isValid() const;

    QString getIncomingServerStr() const;
    QString getOutgoingServerStr() const;
    QString getEmail() const;
    void setEmail(const QString &email);
    void setName(const QString &name);
    void setUsernameFunc(QString user, bool save);
    void setPasswordFunc(QString pass, bool save);

signals:
    void incomingProxyModelChanged();
    void draftsProxyModelChanged();
    void sentProxyModelChanged();
    void archiveProxyModelChanged();
    void trashProxyModelChanged();

    void emailChanged();
    void nameChanged();
    void statusChanged(const Status &status);
    void unseenLettersChanged();

    void queryGetMail();
    void queryUsername();
    void queryPassword();
    void setUsername(QString);
    void setPassword(QString);
    void error(QString);
    void authFailed(QString);
    void sendLetter(Letter *);

    void receiveLetter(Letter *);

public slots:
    void queryIncomingMail();
    void setIncomingServer(QObject *incomingServer);
    void setOutgoingServer(QObject *outgoingServer);
    void setReadyStatus();
    void sendNewLetter();
    void saveNewLetterToDrafts();
    void createAttachment(const QString &path);

    void errorSlot(QString e);
    void authFailedSlot(QString e);

    void onIncomingServerAdressChanged(QString address);
    void onOutgoingServerAdressChanged(QString address);

    void onGetMailIntervalChanged(int interval);

private slots:
    void onUnseenLettersChanged(int c);

private:
    void initializeCache();

    bool    m_initialized;
    bool    m_inPasswordPresent;
    int     m_unseenLetters;
    QString m_email;
    QString m_name;
    QString m_incomingServerStr;
    QString m_outgoingServerStr;
    Status  m_status;
    QTimer  m_refreshTimer;

    Model       * m_incomingModel;
    Model       * m_draftsModel;
    Model       * m_sentModel;
    Model       * m_archiveModel;
    Model       * m_trashModel;

    ProxyModel  * m_incomingProxyModel;
    ProxyModel  * m_draftsProxyModel;
    ProxyModel  * m_sentProxyModel;
    ProxyModel  * m_archiveProxyModel;
    ProxyModel  * m_trashProxyModel;

    Cache       * m_cache;
    Connection  * m_connection;
    Letter      * m_newLetter;
};

}

#endif // PROFILE_H
