#ifndef SERVER_H
#define SERVER_H

#include <QString>
#include <QMutex>
#include <vmime/vmime.hpp>
#include "servermanager.h"
//#include "authenticator_p.h"

namespace FlyMail {

class Authenticator;

class FLYMAILSHARED_EXPORT Server : public QObject
{
    Q_OBJECT
    Q_PROPERTY(QString text READ address WRITE setAddress NOTIFY addressChanged) // for convenience
    Q_PROPERTY(ServerManager::Type type READ type WRITE setType NOTIFY typeChanged)
    Q_PROPERTY(ServerManager::Protocol protocol READ protocol WRITE setProtocol NOTIFY protocolChanged)
    Q_PROPERTY(ServerManager::Ssl ssl READ ssl WRITE setSsl NOTIFY sslChanged)
    Q_PROPERTY(ServerManager::Auth auth READ auth WRITE setAuth NOTIFY authChanged)
    Q_PROPERTY(QString address READ address WRITE setAddress NOTIFY addressChanged)
    Q_PROPERTY(QString login READ login WRITE setLogin NOTIFY loginChanged)

public:
    explicit Server(QObject *parent = nullptr);
    //Server(const Server & server, QObject *parent = nullptr);
    ~Server();

    void setSession(const vmime::shared_ptr<vmime::net::session> &session);
    void setAuthenticator(const vmime::shared_ptr<Authenticator> &auth);

    bool isConnected();

    virtual bool sendEmail(vmime::shared_ptr<vmime::message> &message) = 0;
    virtual bool getEmail(int index = 0) = 0;
    virtual void deleteEmail(int index) = 0;

    ServerManager::Type type();
    void setType(ServerManager::Type type);

    ServerManager::Protocol protocol();
    void setProtocol(ServerManager::Protocol protocol);

    ServerManager::Ssl ssl();
    void setSsl(ServerManager::Ssl ssl);

    ServerManager::Auth auth();
    void setAuth(ServerManager::Auth auth);

    QString address();
    void setAddress(QString address);

    QString login();
    void setLogin(QString login);

    QString port() const;
    bool isSsl() const;

    bool needReconfigure() const;

    bool getSilenceMode() const;
    void setSilenceMode(bool silenceMode);

public slots:
    void onCertificateAnswer(bool answer);
    void testConnection();

    virtual Server * clone() = 0;

signals:
    void typeChanged();
    void protocolChanged();
    void sslChanged();
    void authChanged();
    void addressChanged();
    void addressChanged(QString); // for profile
    void loginChanged();

    void queryCertificate(QString);
    void userAnswer(QString, bool);

    void newMessageId(int, QString);
    void newMessage(vmime::shared_ptr <vmime::net::message>);

    void testConnectionResult(bool);
    void authFailed(QString);

protected:
    vmime::shared_ptr <vmime::net::store> getStore();
    vmime::shared_ptr <vmime::net::transport> getTransport();

    void authenticationFailed();

    ServerManager::Ssl m_connectionType;

private:
    virtual bool connect() = 0;

    QString getUrlString();
    void updateAuthenticator();

    vmime::shared_ptr <vmime::net::session> m_session;
    vmime::shared_ptr <vmime::net::service> m_service;
    vmime::shared_ptr <Authenticator>       m_authenticator;

    ServerManager::Type     m_type;
    ServerManager::Protocol m_protocol;
    ServerManager::Ssl      m_ssl;
    ServerManager::Auth     m_auth;
    QString m_address;
    QString m_login;
    QString m_port;

    QMutex m_typeMutex;
    QMutex m_protocolMutex;
    QMutex m_sslMutex;
    QMutex m_authMutex;
    QMutex m_addressMutex;
    QMutex m_loginMutex;

    bool m_isSsl;
    bool m_needReconfigure;
    bool m_silenceMode {false};
};

}

#endif // SERVER_H
